import os

from reformat_money.settings import EXTENSION


def get_files(dirname):
    files = []
    for basename in os.listdir(dirname):
        filename = os.path.join(dirname, basename)
        if os.path.isfile(filename) and filename.endswith(EXTENSION):
            files.append(filename)
        elif os.path.isdir(filename):
            files.extend(get_files(filename))
    return files



import re

from reformat_money.match import TextMatch


class TextFormatter:
    """Formats the amount using simple regex to determine the starting type."""
    amount_as_string = re.compile(r"^[\"\'](?P<integral>\d+)\.?(?P<decimal>\d*)[\"\']$")  # matches ("120.00", "120", '493.23', '392')
    amount_as_integer = re.compile(r"^(?P<integral>[0-9]+)$")  # matches: (100, 200, 593)
    amount_as_float = re.compile(r"^(?P<integral>\d+)\.(?P<decimal>\d*)$")  # matches: (100., 100.3, 200.32, etc.)

    def __init__(self, match: TextMatch):
        self._match = match
        self._func = self._null_operator

    def match_string_representation(self):
        # Example: Money("1002.39", "GBP")
        found = self.amount_as_string.search(self._match.get_amount())
        if found:
            self._func = self.reformat_string
        return found

    def match_integer_representation(self):
        # Example: Money(200, "GBP")
        found = self.amount_as_integer.search(self._match.get_amount())
        if found:
            self._func = self.reformat_integer
        return found

    def match_float_representation(self):
        # Example: Money(203.23, "GBP")
        found = self.amount_as_float.search(self._match.get_amount())
        if found:
            self._func = self.reformat_float
        return found

    def reformat(self):
        self._inner_match = (
            self.match_string_representation()
            or self.match_integer_representation()
            or self.match_float_representation()
        )

    def _null_operator(self):
        # Method that just doesn't do anything (fun).
        pass

    def reformat_string(self):
        integral = self._inner_match.group("integral")
        decimal = self._inner_match.group("decimal")
        reformatted = self._build_reformatted_amount(integral, decimal)
        self._match.update_amount(reformatted)

    def reformat_integer(self):
        integral = self._inner_match.group("integral")
        reformatted = self._build_reformatted_amount(integral)
        self._match.update_amount(reformatted)

    def reformat_float(self):
        integral = self._inner_match.group("integral")
        decimal = self._inner_match.group("decimal")
        reformatted = self._build_reformatted_amount(integral, decimal)
        self._match.update_amount(reformatted)

    def _build_reformatted_amount(self, integral: str, decimal: str = "00"):
        """Ensure that we have at least 2 decimal places for consistency."""
        while len(decimal) < 2:
            decimal += "0"
        return f'"{integral}.{decimal}"'

    def get_reformatted(self):
        self._func()
        return str(self._match)

    def span(self):
        return (self._match.start, self._match.stop)

    @property
    def able_to_format(self) -> bool:
        return bool(self._inner_match)



import argparse
import os

from reformat_money.file_formatter import FileFormatter
from reformat_money.file_searcher import get_files
from reformat_money.settings import EXTENSION


def parse_args_and_reformat():
    args = get_args()
    reformatter(**args)


def get_args() -> dict:  # pragma: no cover
    """Command line argument parser."""
    parser = argparse.ArgumentParser()
    parser.add_argument(
        "path",
        type=str,
    )
    args = parser.parse_args()
    return {
        "path": args.path,
    }


def reformatter(path):
    """Recursively get all python files and run reformatting."""

    if os.path.isfile(path) and path.endswith(EXTENSION):
        files = [path]
    elif os.path.isdir(path):
        files = get_files(path)
    else:
        raise RuntimeError(
            f"path provided is not a directory or file, or does not have the correct extension: {path}"
        )
    for file in files:
        formatter = FileFormatter(file)
        formatter.reformat()



from typing import List

from reformat_money.arguments import Argument


class TextMatch:
    def __init__(self, args: List[str], start: int, stop: int):
        self.args: List[Argument] = [Argument(arg) for arg in args]
        self.start: int = start
        self.stop: int = stop

    def __str__(self):
        return ",".join([str(arg) for arg in self.args])

    def __repr__(self):
        return f"TextMatch(args={self.args}, start={self.start}, stop={self.stop})"

    def __eq__(self, other):
        return (
            isinstance(other, TextMatch) and
            self.args == other.args and
            self.start is other.start and
            self.stop is other.stop
        )

    def update_amount(self, amount: str):
        self.args[0].set_text(amount)

    def get_amount(self):
        return self.args[0].get_text()





class Argument:
    """Parses a python argument as string.

    All whitespace before and after the argument text itself
    is stripped off and saved for later reformatting.

    """

    def __init__(self, original: str):
        original.lstrip()
        start = 0
        end = len(original) - 1
        while original[start].isspace():
            start += 1
        while original[end].isspace():
            end -= 1
        # End requires +1 when used in string slicing.
        end += 1
        self._leading_whitespace = original[:start]
        self._trailing_whitespace = original[end:]
        self.set_text(original[start:end or None])

    def get_leading_whitespace(self):
        return self._leading_whitespace

    def get_trailing_whitespace(self):
        return self._trailing_whitespace

    def set_text(self, text):
        self._text = text

    def get_text(self):
        return self._text

    def __str__(self):
        return f"{self.get_leading_whitespace()}{self.get_text()}{self.get_trailing_whitespace()}"

    def __repr__(self):
        return f"'{self}'"

    def __eq__(self, other):
        return (
            isinstance(other, Argument) and
            self.get_leading_whitespace() == other.get_leading_whitespace() and
            self.get_text() == other.get_text() and
            self.get_trailing_whitespace() == other.get_trailing_whitespace()
        )


#!/usr/bin/env python

import setuptools


application_dependencies = []
prod_dependencies = []
test_dependencies = [
    'pytest',
    'pytest-cov',
    'pytest-env',
    'ipdb',
]
lint_dependencies = [
    'flake8',
    'flake8-docstrings',
    'black',
]
deploy_dependenies = [
    'setuptools',
    'wheel',
    'twine',
]
docs_dependencies = []
dev_dependencies = test_dependencies + lint_dependencies + docs_dependencies + deploy_dependenies + []


with open('README.md', 'r') as fh:
    long_description = fh.read()


setuptools.setup(
    name='reformat-money',
    version='0.0.1',
    author='Mike Wooster',
    author_email='',
    description='A script to fix money references within a codebase.',
    long_description=long_description,
    long_description_content_type='text/markdown',
    url='https://github.com/MikeWooster/reformat-money',
    python_requires='>=3.6',
    packages=[
        'reformat_money',
    ],
    classifiers=[
        'Development Status :: 2 - Pre-Alpha',
        'Programming Language :: Python :: 3.6',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Intended Audience :: Developers',
    ],
    install_requires=application_dependencies,
    extras_require={
        'production': prod_dependencies,
        'test': test_dependencies,
        'lint': lint_dependencies,
        'docs': dev_dependencies,
        'dev': dev_dependencies,
    },
    include_package_data=True,
    zip_safe=False,
    entry_points={
        'console_scripts': [
            'reformat-money=reformat_money.__main__:main',
        ]
    },
)



import re
from typing import List

from reformat_money.match import TextMatch
from reformat_money.text_formatter import TextFormatter


class FileFormatter:
    SEARCH_PATTERN = re.compile(r"Money\(")
    REFORMAT_PATTERN = re.compile(r"(?P<class_name>Money)\((?P<contents>.*[\"\']\w{3}[\"\'])\)")
    CLOSING_BRACKET = ")"
    OPENING_BRACKET = "("
    QUOTES = {"'", '"',}
    ESCAPE = "\\"
    COMMA = ","

    def __init__(self, file):
        self._file = file
        self._total_matches = 0
        self._formatted = 0
        self._unable_to_format = 0

    def reformat(self):
        with open(self._file, 'r') as buf:
            self.set_contents(buf.read())

        self.find_and_reformat_matches()

        with open(self._file, 'w') as buf:
            buf.write(self.get_contents())

    def set_contents(self, contents: str):
        self._contents_str = contents
        self._contents = list(contents)
        self._file_characters = len(self._contents)

    def get_contents(self) -> str:
        return "".join(self._contents)

    def find_and_reformat_matches(self):
        for match in self.find_matches():
            self._reformat_match(match)
        self._maybe_print_summary()

    def find_matches(self) -> List[TextMatch]:
        """Returns list of matches.

        Matches are returned in the reverse order that
        they appear in file.
        """
        matches = []
        for match in self.SEARCH_PATTERN.finditer(self._contents_str):
            self._total_matches += 1
            start = match.end()
            text_match = self.split_args(start)
            matches.append(text_match)
        return matches[::-1]

    def split_args(self, start: int) -> TextMatch:
        inspect = True
        opening_quote = "'"
        opening_bracket_count = 1
        closing_bracket_count = 0
        argument_start_positions = []
        stop = None

        for i in range(start, self._file_characters):
            char = self._contents[i]
            prev_char = self._contents[i - 1] if i >= 1 else None
            # Ignore everything that appears within quotes.
            # TODO: handle triple quotes
            if inspect and char in self.QUOTES:
                opening_quote = char
                inspect = False
                continue
            if not inspect and char == opening_quote and prev_char != self.ESCAPE:
                inspect = True
                continue
            if inspect and char == self.OPENING_BRACKET:
                opening_bracket_count += 1
            if inspect and (opening_bracket_count - closing_bracket_count) == 1 and char == self.COMMA:
                # This denotes a new argument starting
                argument_start_positions.append(i)
            if inspect and char == self.CLOSING_BRACKET:
                closing_bracket_count += 1
                if closing_bracket_count == opening_bracket_count:
                    # Stop position should not include the closing bracket.
                    stop = i - 1
                    break
        if stop is None:
            raise RuntimeError(f"Unable to find closing bracket for start position '{start}'")
        if argument_start_positions:
            # Append the last position so that we include the last argument too.
            # As we will be using slicing, we need to increment stop by one.
            argument_start_positions.append(stop+1)
            match = TextMatch(
                args=self._get_arguments_from_start_positions(start, argument_start_positions),
                start=start,
                stop=stop,
            )
        else:
            match = TextMatch(
                args=["".join(self._contents[start: stop+1])],
                start=start,
                stop=stop,
            )
        return match

    def _get_arguments_from_start_positions(self, last_pos: int, positions: list) -> List[str]:
        args = []
        for pos in positions:
            args.append("".join(self._contents[last_pos: pos]))
            last_pos = pos + 1
        return args

    def _reformat_match(self, match: TextMatch):
        original_amount = match.get_amount()
        formatter = TextFormatter(match)
        formatter.reformat()

        self._update(formatter)

        if not formatter.able_to_format:
            self._unable_to_format += 1
        elif original_amount != match.get_amount():
            self._formatted += 1

    def _maybe_print_summary(self):
        if self._total_matches and self._unable_to_format == 0 and self._formatted > 0:
            print(f"{self._file} -- Reformatted: {self._formatted}")
        elif self._total_matches and self._unable_to_format > 0:
            print(f"{self._file} -- Reformatted: {self._formatted}, Unable to reformat: {self._unable_to_format}")

    def _update(self, formatter: TextFormatter):
        start, end = formatter.span()
        new_string = formatter.get_reformatted()
        del (self._contents[start:end+1])
        pos = start
        for i, char in enumerate(new_string):
            self._contents.insert(pos+i, char)
